# -*- coding: utf-8 -*-

#4a) Si scriva una "generator function" allMRNAs(pseq) che, considerata la 
# lista ordinata lessicograficamente di tutte le stringhe corrispondenti
# alle sequenze di mRNA che danno luogo a pseq, consenta di ottenere un 
# iteratore per scandire tale lista.


from gencodes import code_gen

ordered_code_gen={}
for key in code_gen:
    ordered_code_gen[key]= sorted(code_gen[key])
    
   
def allMRNAs(pseq):
    """
    Generator function to obtain a generator for the mRNAs that yield pseq.
    """

    allCodons = [ordered_code_gen[aa] for aa in pseq]
    currEncoding = [0 for aa in pseq]
        
    while True:
        yield "".join(c[i] for c, i in zip(allCodons, currEncoding))
        #update currEncoding for the next seq to generate
        for pos in xrange(len(pseq)-1,-1,-1):
            res = currEncoding[pos]+1
            if res < len(allCodons[pos]):
                currEncoding[pos] = res
                break
            elif pos==0:
                raise StopIteration
            else:
                currEncoding[pos] = 0


def allMRNAs2(pseq):
    """
    Generator function to obtain a generator for the mRNAs that yield pseq.
    Variation of the previous solution with recursive update.
    """

    allCodons = [ordered_code_gen[aa] for aa in pseq]
    currEncoding = [0 for aa in pseq]
    
    def nextEncoding():
        #inner function to update currEncoding for the next seq to generate
        def incPosition(pos):
            res = currEncoding[pos]+1
            if res < len(allCodons[pos]):
                currEncoding[pos] = res
            elif pos==0:
                raise StopIteration
            else:
                currEncoding[pos] = 0
                incPosition(pos-1)
        
        incPosition(len(pseq)-1)
    
    while True:
        yield "".join(c[i] for c, i in zip(allCodons, currEncoding))
        nextEncoding()
        
